#include "renderer_api.h"
#include "mem_arena.h"
#include "handles.h"

typedef struct {
    rt_pipeline_handle pipeline;
} rt_object_renderer;

static rt_object_renderer _object_renderer;

#define PIPELINE_ID 0xdee414bba9b4f5bdLL

rt_result InitObjectRenderer(void) {
    rt_result result = RT_SUCCESS;
    rt_temp_arena temp = rtGetTemporaryArena(NULL, 0);
    if (!temp.arena) {
        result = RT_OUT_OF_MEMORY;
        goto out;
    }

    /* Init the pipeline */
    size_t pipeline_size = rtGetResourceSize(PIPELINE_ID);
    if (pipeline_size == 0) {
        rtReportError("GFX", "Failed to determine size of object pipeline %llx", PIPELINE_ID);
        result = RT_INVALID_VALUE;
        goto out;
    }
    rt_resource *pipeline_resource = rtArenaPush(temp.arena, pipeline_size);
    if (!pipeline_resource) {
        rtReportError("GFX", "Failed to allocate memory for object pipeline %llx", PIPELINE_ID);
        result = RT_OUT_OF_MEMORY;
        goto out;
    }
    result = rtGetResource(PIPELINE_ID, pipeline_resource);
    if (result != RT_SUCCESS) {
        rtReportError("GFX", "Failed to load the object pipeline %llx", PIPELINE_ID);
        goto out;
    }
    rt_pipeline_info *info = pipeline_resource->data;
    if (!info) {
        rtReportError("GFX", "Malformed object pipeline %llx (missing pipeline_info)", PIPELINE_ID);
        result = RT_INVALID_VALUE;
        goto out;
    }
    _object_renderer.pipeline = g_renderer.CompilePipeline(info);
    if (!RT_IS_HANDLE_VALID(_object_renderer.pipeline)) {
        rtReportError("GFX", "Failed to compile the object pipeline %llx", PIPELINE_ID);
        result = RT_UNKNOWN_ERROR;
        goto out;
    }

out:
    rtReturnTemporaryArena(temp);
    return result;
}

void ShutdownObjectRenderer(void) {
    g_renderer.DestroyPipeline(_object_renderer.pipeline);
}